#![cfg_attr(
    all(not(debug_assertions), target_os = "windows"),
    windows_subsystem = "windows"
)]
#![feature(bool_to_option)]

mod core;
mod db;
mod display;
mod picker;
mod rpc;
mod models;

use crate::core::AmbientLightMode;
use crate::core::CoreManager;
use once_cell::sync::OnceCell;
use paris::*;
use picker::config::DisplayConfig;
use picker::manager::Picker;
use picker::screenshot::ScreenshotDto;
use tauri::async_runtime::Mutex;

static GET_SCREENSHOT_LOCK: OnceCell<Mutex<bool>> = OnceCell::new();

#[tauri::command]
async fn take_snapshot() -> Vec<ScreenshotDto> {
    info!("Hi?");
    let _lock = GET_SCREENSHOT_LOCK
        .get_or_init(|| Mutex::new(false))
        .lock()
        .await;
    info!("Hi!");
    let manager = Picker::global().await;

    let start = time::Instant::now();
    let base64_bitmap_list = match manager.list_displays().await {
        Ok(base64_bitmap_list) => {
            info!("screenshots len: {}", base64_bitmap_list.len());
            base64_bitmap_list
        }
        Err(error) => {
            error!("can not take screenshots for all. {}", error);

            vec![]
        }
    };
    info!("截图耗时 {} s", start.elapsed().as_seconds_f32());
    base64_bitmap_list
}

#[tauri::command]
async fn get_screenshot_by_config(config: DisplayConfig) -> Result<ScreenshotDto, String> {
    info!("Hi?");
    // let _lock = GET_SCREENSHOT_LOCK.get_or_init(|| Mutex::new(false)).lock().await;
    // info!("Hi!");
    let start = time::Instant::now();
    let screenshot_dto = Picker::preview_display_by_config(config).await;
    info!("截图耗时 {} s", start.elapsed().as_seconds_f32());
    match screenshot_dto {
        Ok(screenshot_dto) => Ok(screenshot_dto),
        Err(error) => {
            error!("preview_display_by_config failed. {}", error);
            Err(format!("preview_display_by_config failed. {}", error))
        }
    }
}

#[tauri::command]
async fn get_picker_config() -> picker::config::Configuration {
    let configuration = picker::config::Manager::global().get_config().await;
    info!("configuration: {:?}", configuration);
    configuration
}
#[tauri::command]
async fn write_picker_config(config: picker::config::Configuration) -> Result<(), String> {
    let manager = picker::config::Manager::global();
    let path = picker::config::Manager::get_config_file_path();
    info!("log save in {:?}", path.to_str());
    manager.set_config(&config).await;
    match picker::config::Manager::write_config_to_disk(path, &config) {
        Ok(_) => Ok(()),
        Err(err) => {
            error!("can not write picker config. {:?}", err);
            Err(format!("can not write picker config. {:?}", err))
        }
    }
}

#[tauri::command]
async fn play_mode(target_mode: AmbientLightMode) {
    info!("target mode: {:?}", target_mode);

    tokio::spawn(async move { CoreManager::global().set_ambient_light(target_mode).await });
}

#[tokio::main]
async fn main() {
    let display_manager = display::Manager::global();
    tokio::spawn(display_manager.subscribe_display_brightness());
    let rpc_manager = rpc::Manager::global().await;
    tokio::spawn(rpc_manager.listen());

    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![
            take_snapshot,
            play_mode,
            get_picker_config,
            get_screenshot_by_config,
            write_picker_config,
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
